SystemOrganization addCategory: #LostInMorvanChat!
SystemOrganization addCategory: 'LostInMorvanChat-Client'!
SystemOrganization addCategory: 'LostInMorvanChat-Server'!

ComposableModel subclass: #ChatConsole
	instanceVariableNames: 'chat list input'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LostInMorvanChat-Client'!

!ChatConsole class methodsFor: 'api' stamp: 'FrancoisJarriges 6/1/2016 11:40'!
attach: aChatClient

	| aChatConsole |
	
	aChatConsole := self new chat: aChatClient.
	aChatConsole openWithSpec whenClosedDo: [ aChatClient disconnect ].
	^ aChatConsole! !

!ChatConsole class methodsFor: 'api' stamp: 'FrancoisJarriges 6/1/2016 11:53'!
defaultSpec
	<spec: #default>
	
	^ SpecLayout composed
			newColumn: [ :c |
				c add: #list; add: #input height: 30 ]; yourself
! !

!ChatConsole methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/1/2016 11:32'!
chat: aChatClient
	chat := aChatClient! !

!ChatConsole methodsFor: 'api' stamp: 'FrancoisJarriges 6/1/2016 11:58'!
initializeWidgets

	list := ListModel new.
	input := TextInputFieldModel new
		ghostText: 'Type your message here...';
		enabled: true;
		acceptBlock: [ :inputString |
			chat send: inputString.
			input text: '' ].
	self focusOrder add: input
! !

!ChatConsole methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/1/2016 11:31'!
input
	^ input! !

!ChatConsole methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/1/2016 11:31'!
list
	^ list! !

!ChatConsole methodsFor: 'api' stamp: 'FrancoisJarriges 6/1/2016 12:05'!
print: aCollectionOfChatMessages

	list items: (aCollectionOfChatMessages collect: [ :eachChatMessage | 
		eachChatMessage printString ])
! !

!ChatConsole methodsFor: 'api' stamp: 'FrancoisJarriges 6/1/2016 11:33'!
title
	^ 'Lost In Morvan Chat'! !

Object subclass: #ChatClient
	instanceVariableNames: 'url login exit messages console lastMessageIndex networkError'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LostInMorvanChat-Client'!

!ChatClient class methodsFor: 'public' stamp: 'FrancoisJarriges 6/1/2016 11:19'!
connect: aHost port: aPort login: aLogin

	^ self new
		host: aHost port: aPort login: aLogin;
		start
! !

!ChatClient methodsFor: 'private' stamp: 'FrancoisJarriges 6/1/2016 19:28'!
basicSecuredGet: commandString
	
	| responseString |
	
	[ responseString := ZnClient new 
		url: commandString; 
		get ] 
		on: NetworkError 
		do: [ :thisNetworkError | 
			self printConnectionErrorMessage: thisNetworkError asString.
			responseString := nil ].
	responseString ifNotNil: [ networkError := nil ].
	^ responseString
! !

!ChatClient methodsFor: 'private' stamp: 'FrancoisJarriges 6/1/2016 19:26'!
basicSecuredPost: commandString sender: senderString text: textString

	| responseString |
	
	[ responseString := ZnClient new
		url: commandString;
		formAt: 'sender' put: senderString;
		formAt: 'text' put: textString;
		post ]
		on: NetworkError 
		do: [ :thisNetworkError | 
			self printConnectionErrorMessage: thisNetworkError asString.
			responseString := nil ].
	responseString ifNotNil: [ networkError := nil ].
	^ responseString ! !

!ChatClient methodsFor: 'private' stamp: 'FrancoisJarriges 6/1/2016 10:30'!
command: aPath

	^ '{1}{2}' format: { url . aPath }! !

!ChatClient methodsFor: 'private' stamp: 'FrancoisJarriges 6/1/2016 10:30'!
command: aPath argument: anArgument

	^ '{1}{2}{3}' format: { url . aPath . anArgument asString }! !

!ChatClient methodsFor: 'private' stamp: 'FrancoisJarriges 6/1/2016 10:37'!
commandAddMessage

	^ self command: '/messages/add'! !

!ChatClient methodsFor: 'private' stamp: 'FrancoisJarriges 6/1/2016 10:34'!
commandLastMessageIndex

	^ self command: '/messages/count'! !

!ChatClient methodsFor: 'private' stamp: 'FrancoisJarriges 6/1/2016 14:31'!
commandMessagesFromLastIndex

	^ self command: '/messages/' argument: lastMessageIndex ! !

!ChatClient methodsFor: 'public' stamp: 'FrancoisJarriges 6/1/2016 11:13'!
disconnect

	self postNewMessage: (ChatMessage from: login text: 'I exited from the chat room.').
	exit := true

! !

!ChatClient methodsFor: 'private' stamp: 'FrancoisJarriges 6/1/2016 18:22'!
getLastMessageIndexAtStart

	| responseString |	
			
	[ responseString := self basicSecuredGet: self commandLastMessageIndex ] value
		ifNil: [ ^ lastMessageIndex ].
	^ responseString asInteger 
! !

!ChatClient methodsFor: 'private' stamp: 'FrancoisJarriges 6/1/2016 17:18'!
getNewMessages
	"Gets the new messages that have been posted since the last request."
	"Gets rid of any uncompliant messages from the server."

	| response receivedMessageStrings |
	
	^ [ response := self basicSecuredGet: self commandMessagesFromLastIndex ] value
		ifNil: [ 0 ]
		ifNotNil: [
			receivedMessageStrings := (response substrings: String crlf) 
				select: [ :eachMessageString |
					
					| newChatMessage |
					
					(newChatMessage := ChatMessage fromString: eachMessageString ) 
						ifNotNil: [ messages add: newChatMessage ].
					newChatMessage isNotNil ].
			receivedMessageStrings size	 ]
	
! !

!ChatClient methodsFor: 'initialization' stamp: 'FrancoisJarriges 6/1/2016 11:22'!
host: aHost port: aPort login: aLogin

	url := 'http://' , aHost , ':' , aPort asString.
	login := aLogin

! !

!ChatClient methodsFor: 'initialization' stamp: 'FrancoisJarriges 6/1/2016 18:52'!
initialize

	super initialize.
	exit := false.
	networkError := nil.
	lastMessageIndex := 1.
	messages := OrderedCollection new! !

!ChatClient methodsFor: 'private' stamp: 'FrancoisJarriges 6/1/2016 17:15'!
postNewMessage: aChatMessage

	^ self basicSecuredPost: self commandAddMessage
		sender: aChatMessage sender
		text: aChatMessage text
! !

!ChatClient methodsFor: 'private' stamp: 'FrancoisJarriges 6/1/2016 19:06'!
printConnectionErrorMessage: aNetworkErrorString

	| errorMessage |
	
	aNetworkErrorString = networkError ifFalse: [ 
		networkError := aNetworkErrorString.
		errorMessage := ChatMessage 
			from: 'Network Error' 
			text: networkError.
		messages add: errorMessage ]


	! !

!ChatClient methodsFor: 'private' stamp: 'FrancoisJarriges 6/1/2016 11:00'!
refreshMessages
	"Create and schedule a light process to get new messages every 2 seconds."
	"When the exit instance variable is set to true the process ends."
	
	[
		[ exit ] whileFalse: [
			(Delay forSeconds: 2) wait.
			lastMessageIndex := lastMessageIndex + self getNewMessages.
			console print: messages ] 
	] fork! !

!ChatClient methodsFor: 'public' stamp: 'FrancoisJarriges 6/1/2016 11:11'!
send: aString
	"When we send a message, we post it to the server and in addition we update the local list of posted messages."
	
	| newChatMessage |
	
	newChatMessage := ChatMessage from: login text: aString.
	self postNewMessage: newChatMessage.
	lastMessageIndex := lastMessageIndex + self getNewMessages.
	console print: messages
! !

!ChatClient methodsFor: 'public' stamp: 'FrancoisJarriges 6/1/2016 15:38'!
start

	console := ChatConsole attach: self.
	self postNewMessage: (ChatMessage from: login text: 'I joined the chat room').
	lastMessageIndex := self getLastMessageIndexAtStart.
	self refreshMessages! !

Object subclass: #ChatMessage
	instanceVariableNames: 'sender text'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LostInMorvanChat'!

!ChatMessage class methodsFor: 'instance-creation' stamp: 'FrancoisJarriges 6/1/2016 08:59'!
from: aSender text: aText

	^ self new sender: aSender; text: aText; yourself
! !

!ChatMessage class methodsFor: 'instance-creation' stamp: 'FrancoisJarriges 6/1/2016 16:55'!
fromString: aString
	"Verify the string conformity before creating a new ChatMessage from it."

	(aString substrings: self separator) size = 2
		ifFalse: [ 
			Notification signal: 'Chat application unable to treat this message: ', aString.
			^ nil ].
	^ self new 
		fromString: aString;
		yourself
! !

!ChatMessage class methodsFor: 'api' stamp: 'FrancoisJarriges 6/1/2016 12:54'!
separator
	^ '>'! !

!ChatMessage methodsFor: 'instance creation' stamp: 'FrancoisJarriges 6/1/2016 16:54'!
fromString: aString
	"Compose a message from a string of this form 'sender>message'."
	| items |
	items := aString substrings: self class separator.
	self sender: items first.
	self text: items second
! !

!ChatMessage methodsFor: 'printing' stamp: 'FrancoisJarriges 6/1/2016 15:01'!
printOn: aStream

	aStream 
		<< self sender; 
		<< self class separator;
		<< String space;
		<< self text; 
		<< String crlf! !

!ChatMessage methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/1/2016 08:57'!
sender
	^ sender! !

!ChatMessage methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/1/2016 08:57'!
sender: anObject
	sender := anObject! !

!ChatMessage methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/1/2016 08:57'!
text
	^ text! !

!ChatMessage methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/1/2016 08:57'!
text: anObject
	text := anObject! !

Object subclass: #ChatMessagesQueue
	instanceVariableNames: 'messages'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LostInMorvanChat-Server'!

!ChatMessagesQueue methodsFor: 'adding-removing' stamp: 'FrancoisJarriges 6/1/2016 09:12'!
add: aChatMessage
	messages add: aChatMessage ! !

!ChatMessagesQueue methodsFor: 'converting' stamp: 'FrancoisJarriges 6/1/2016 09:21'!
formattedMessagesFrom: aChatMessageIndex

	^ String streamContents: [ :formattedMessagesStream |
		(self listFrom: aChatMessageIndex) do: [ :eachChatMessage | 
			formattedMessagesStream << eachChatMessage printString ] ]! !

!ChatMessagesQueue methodsFor: 'initialization' stamp: 'FrancoisJarriges 6/1/2016 09:11'!
initialize

	super initialize.
	messages := OrderedCollection new! !

!ChatMessagesQueue methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/1/2016 09:23'!
listFrom: aChatMessageIndex

	^ (aChatMessageIndex > 0 and: [ aChatMessageIndex <= messages size])
		ifTrue: [ messages copyFrom: aChatMessageIndex to: messages size ]
		ifFalse: [ #() ]! !

!ChatMessagesQueue methodsFor: 'adding-removing' stamp: 'FrancoisJarriges 6/1/2016 09:12'!
reset
	messages removeAll! !

!ChatMessagesQueue methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/1/2016 09:12'!
size
	^ messages size! !

Object subclass: #ChatServer
	instanceVariableNames: 'teapotServer messagesQueue'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LostInMorvanChat-Server'!

!ChatServer class methodsFor: 'public' stamp: 'FrancoisJarriges 6/1/2016 09:39'!
startOn: aPortNumber

	^ self new
		initializePort: aPortNumber;
		registerRoutes;
		registerErrorHandlers;
		yourself! !

!ChatServer class methodsFor: 'public' stamp: 'FrancoisJarriges 6/1/2016 09:44'!
stopAll

	^ self allInstancesDo: #stop! !

!ChatServer methodsFor: 'requests' stamp: 'FrancoisJarriges 6/1/2016 09:51'!
addMessage: aRequest

	messagesQueue add: (ChatMessage 
		from: (aRequest at: #sender) 
		text:	(aRequest at: #text))! !

!ChatServer methodsFor: 'initialization' stamp: 'FrancoisJarriges 6/1/2016 09:28'!
initialize

	super initialize.
	messagesQueue := ChatMessagesQueue new! !

!ChatServer methodsFor: 'initialization' stamp: 'FrancoisJarriges 6/1/2016 09:30'!
initializePort: anInteger

	teapotServer := Teapot configure: {
		#defaultOutput -> #text.
		#port -> anInteger.
		#debugMode -> true }.
	teapotServer start! !

!ChatServer methodsFor: 'requests' stamp: 'FrancoisJarriges 6/1/2016 09:52'!
messageCount

	^ messagesQueue size! !

!ChatServer methodsFor: 'requests' stamp: 'FrancoisJarriges 6/1/2016 14:17'!
messagesFrom: aRequest

	^ messagesQueue formattedMessagesFrom: (aRequest at: #id)! !

!ChatServer methodsFor: 'initialization' stamp: 'FrancoisJarriges 6/1/2016 10:12'!
registerErrorHandlers
	
	teapotServer 
		exception: KeyNotFound -> (TeaResponse notFound body: 'No such message')! !

!ChatServer methodsFor: 'initialization' stamp: 'FrancoisJarriges 6/1/2016 14:17'!
registerRoutes
	
	teapotServer 
		GET: '/messages/count' -> (Send message: #messageCount to: self);
		GET: '/messages/<id:IsInteger>' -> (Send message: #messagesFrom: to: self);
		POST: '/messages/add' -> (Send message: #addMessage: to: self)! !

!ChatServer methodsFor: 'public' stamp: 'FrancoisJarriges 6/1/2016 09:41'!
stop

	teapotServer stop.
	messagesQueue reset! !
